#include "EditWindow.h"
#include <wx/dc.h>
#include <wx/brush.h>

EditWindow::EditWindow(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size, long style, const wxString &name)
	: wxScrolled(parent, id, pos, size, style, name), widthInTiles(64), heightInTiles(64), 
	zoom(2), isLoaded(false), currentDC(nullptr), currentClippedRect(), currentFullRect(),
	currentXScroll(0), currentYScroll(0)
{}

EditWindow::~EditWindow() {}

int EditWindow::GetTileSideLength() const {
	return 8;
}

void EditWindow::SetSizeInTiles() {
	GetTargetWindow()->SetVirtualSize(widthInTiles * GetTileSideLength() * GetZoom(), 
		heightInTiles * GetTileSideLength() * GetZoom());
	SetScrollRate(GetTileSideLength() * GetZoom(), GetTileSideLength() * GetZoom());
}

void EditWindow::SetSizeInTiles(int xtiles, int ytiles) {
	widthInTiles = xtiles;
	heightInTiles = ytiles;
	SetSizeInTiles();
}

int EditWindow::GetWidthInTiles() const {
	return widthInTiles;
}

int EditWindow::GetHeightInTiles() const {
	return heightInTiles;
}

void EditWindow::SetZoom(int z) {
	zoom = z;
	SetSizeInTiles();
}

int EditWindow::GetZoom() const {
	return zoom;
}

wxPoint EditWindow::GetTileDisplayStart() const {
	wxPoint pt = GetViewStart();
	pt.x /= GetZoom();
	pt.y /= GetZoom();
	return pt;
}

wxSize EditWindow::GetTileDisplaySize() const {
	wxSize size = GetClientSize();
	wxSize size2(GetWidthInTiles() * GetTileSideLength(), GetHeightInTiles() * GetTileSideLength());
	size.SetWidth(size.GetWidth() / GetZoom() + (size.GetWidth() % GetZoom() == 0 ? 0 : 1));
	size.SetHeight(size.GetHeight() / GetZoom() + (size.GetHeight() % GetZoom() == 0 ? 0 : 1));
	size.SetWidth(size.GetWidth() > size2.GetWidth() ? size2.GetWidth() : size.GetWidth());
	size.SetHeight(size.GetHeight() > size2.GetHeight() ? size2.GetHeight() : size.GetHeight());
	return size;
}

wxRect EditWindow::GetTileDisplayArea() const {
	return wxRect(GetTileDisplayStart(), GetTileDisplaySize());
}

//will not go beyond bounds of visible client area
wxRect EditWindow::GetClippedTileRectangle(int xpos, int ypos) const {
	return GetClippedTileRectangle(wxPoint(xpos, ypos));
}

//will not go beyond bounds of visible client area
wxRect EditWindow::GetClippedTileRectangle(wxPoint pt) const {
	wxPoint origin = GetTileDisplayStart();
	wxSize viewsize = GetTileDisplaySize();
	wxPoint termination(origin.x + viewsize.GetWidth(), origin.y + viewsize.GetHeight());
	pt.x = (origin.x + pt.x) / GetTileSideLength() * GetTileSideLength() - origin.x;
	pt.y = (origin.y + pt.y) / GetTileSideLength() * GetTileSideLength() - origin.y;
	wxPoint pt2(pt.x + GetTileSideLength() - origin.x, pt.y + GetTileSideLength() - origin.y);
	pt.x = pt.x < 0 ? 0 : pt.x;
	pt.y = pt.y < 0 ? 0 : pt.y;
	pt2.x = pt2.x >= termination.x ? termination.x : pt2.x;
	pt2.y = pt2.y >= termination.y ? termination.y : pt2.y;
	return wxRect(pt.x, pt.y, pt2.x - pt.x, pt2.y - pt.y);
}

//may go beyond bounds of visual client area
wxRect EditWindow::GetFullTileRectangle(int xpos, int ypos) const {
	return GetFullTileRectangle(wxPoint(xpos, ypos));
}

//may go beyond bounds of visual client area
wxRect EditWindow::GetFullTileRectangle(wxPoint pt) const {
	wxPoint origin = GetTileDisplayStart();
	pt.x = (origin.x + pt.x) / GetTileSideLength() * GetTileSideLength() - origin.x;
	pt.y = (origin.y + pt.y) / GetTileSideLength() * GetTileSideLength() - origin.y;
	wxPoint pt2(pt.x + GetTileSideLength() - origin.x, pt.y + GetTileSideLength() - origin.y);
	return wxRect(pt.x, pt.y, pt2.x - pt.x, pt2.y - pt.y);
}

//gets index
int EditWindow::GetIndexFromPoint(int xpos, int ypos) const {
	return GetIndexFromPoint(wxPoint(xpos, ypos));
}

//gets index
int EditWindow::GetIndexFromPoint(wxPoint pt) const {
	wxPoint origin = GetTileDisplayStart();
	int tempx = (origin.x + pt.x) / GetTileSideLength();
	int tempy = (origin.y + pt.y) / GetTileSideLength();
	return tempx + GetWidthInTiles() * tempy;
}

//do not use
//reserved for wxScrolled<wxWindow> superclass
void EditWindow::OnDraw(wxDC& dc) {
	if (GetIsLoaded() == false) return;
	wxSize size = GetTileDisplaySize();
	currentDC = &dc;
	for (int y = 0; y < size.GetHeight(); y += GetTileSideLength()) {
		for (int x = 0; x < size.GetWidth(); x += GetTileSideLength()) {
			wxPoint pt(x, y);
			currentClippedRect = GetClippedTileRectangle(pt);
			currentFullRect = GetFullTileRectangle(pt);
			OnDrawATile(GetIndexFromPoint(pt));
		}
	}
	//clean up
	currentDC = nullptr;
	currentClippedRect.SetHeight(0);
	currentFullRect.SetHeight(0);
}

void EditWindow::DrawPixel(int x, int y, const wxColour& color) {
	x += currentFullRect.GetLeft() + GetMargin();
	y += currentFullRect.GetTop() + GetMargin();
	wxPoint pt(x, y);
	if (!FallsWithinClippedRectangle(pt, currentClippedRect)) return;
	pt.x *= GetZoom();
	pt.y *= GetZoom();
	wxRect rect(pt, wxSize(GetZoom(), GetZoom()));
	currentDC->SetPen(*wxThePenList->FindOrCreatePen(color));
	currentDC->SetBrush(*wxTheBrushList->FindOrCreateBrush(color));
	currentDC->DrawRectangle(rect);
}

//only works within OnDrawATile() and functions called from it
void EditWindow::DrawRect(const wxColour& color) {
	wxRect rect(currentClippedRect.GetLeft() * GetZoom(),
		currentClippedRect.GetTop() * GetZoom(),
		currentClippedRect.GetWidth() * GetZoom(),
		currentClippedRect.GetHeight() * GetZoom());
	currentDC->SetBrush(*wxTheBrushList->FindOrCreateBrush(color));
	currentDC->DrawRectangle(rect);
}

//invalidates parent
void EditWindow::Invalidate() {
	currentXScroll = GetScrollPos(wxHORIZONTAL);
	currentYScroll = GetScrollPos(wxVERTICAL)
	GetParent()->Refresh();
}

void EditWindow::SetIsLoaded(bool isSo) {
	isLoaded = isSo;
}

bool EditWindow::GetIsLoaded() const {
	return isLoaded;
}

bool EditWindow::FallsWithinClippedRectangle(const wxPoint& pt, const wxRect& clipped) const {
	if (pt.x < clipped.GetLeft()) return false;
	if (pt.y < clipped.GetTop()) return false;
	if (pt.x >= clipped.GetRight()) return false;
	if (pt.y >= clipped.GetBottom()) return false;
	return true;
}

int EditWindow::GetMargin() const {
	return 0;
}
